local UnitName = UnitName
local UnitGUID = UnitGUID
local IsInRaid = IsInRaid
local IsInGroup = IsInGroup
local GetPartyMember = GetPartyMember
local InCombatLockdown = InCombatLockdown
local PartyMemberFrame_OnLoad = PartyMemberFrame_OnLoad
local IsActiveBattlefieldArena = IsActiveBattlefieldArena
local UpdatePartyMemberBackground = UpdatePartyMemberBackground

function GetDisplayedAllyFrames()
	local useCompact = CUF_CVar:GetCVarBool("useCompactPartyFrames")

	if ( IsActiveBattlefieldArena() and not useCompact ) then
		return "party"
	elseif ( IsInGroup() and ( IsInRaid() or useCompact ) ) then
		return "raid"
	elseif ( IsInGroup() ) then
		return "party"
	else
		return nil
	end
end

local function HidePartyFrame()
	for i=1, MAX_PARTY_MEMBERS, 1 do
		local Frame = _G["PartyMemberFrame"..i]
		Frame:SetAlpha(0)
		Frame:Hide()
		Frame:UnregisterAllEvents()
	end
end

local function ShowPartyFrame()
	for i=1, MAX_PARTY_MEMBERS, 1 do
		if ( GetPartyMember(i) ) then
			local Frame = _G["PartyMemberFrame"..i]
			Frame:SetAlpha(1)
			Frame:Show()
			PartyMemberFrame_OnLoad(Frame)
		end
	end
end

function RaidOptionsFrame_UpdatePartyFrames()
	if ( InCombatLockdown() ) then
		return
	end

	if ( GetDisplayedAllyFrames() ~= "party" ) then
		HidePartyFrame()
	else
		HidePartyFrame()
		ShowPartyFrame()
	end
	UpdatePartyMemberBackground()
end

local LibGT = LibStub:GetLibrary("LibGroupTalents-1.0", true);
local LibResComm = LibStub:GetLibrary("LibResComm-1.0", true);

local function EventHandler(self, event, arg1, arg2, arg3)
	local unit = self.unit or self.displayedUnit;

	if ( UnitExists(unit) ) then
		if ( event == "LibGroupTalents_RoleChange" ) then
			CompactUnitFrame_OnEvent(self, "PLAYER_ROLES_ASSIGNED", unit);
		elseif ( event == "ResComm_ResStart" or event == "ResComm_ResEnd" ) then
			local unitName = UnitName(unit);
			if ( arg3 == unitName or arg2 == unitName ) then
				CompactUnitFrame_OnEvent(self, "INCOMING_RESURRECT_CHANGED", unit);
			end
		end
	end
end

function CompactUnitFrame_RegisterCallback(self)
	if ( LibGT ) then
		LibGT.RegisterCallback(self, "LibGroupTalents_RoleChange", EventHandler, self);
	end

	if ( LibResComm ) then
		--LibResComm.RegisterCallback(self, "ResComm_CanRes", EventHandler, self);
		--LibResComm.RegisterCallback(self, "ResComm_Ressed", EventHandler, self);
		--LibResComm.RegisterCallback(self, "ResComm_ResExpired", EventHandler, self);
		LibResComm.RegisterCallback(self, "ResComm_ResStart", EventHandler, self);
		LibResComm.RegisterCallback(self, "ResComm_ResEnd", EventHandler, self);
	end
end

function CompactRaidFrameManager_RegisterCallback(self)
	LibGT.RegisterCallback(self, "LibGroupTalents_RoleChange", function(self, event, ...)
		if ( event == "LibGroupTalents_RoleChange" ) then
			CompactRaidFrameManager_OnEvent(self, "UPDATE_ACTIVE_BATTLEFIELD");
		end
	end, self);
end

--[[

	Roles

]]

local LibGTConvertRole = {
	["melee"]   = "DAMAGER",
	["caster"]  = "DAMAGER",
	["healer"]  = "HEALER",
	["tank"]    = "TANK",
}

function GetSpecializationRole(specIndex)
	if ( LibGT ) then
		return LibGTConvertRole[LibGT:GetUnitRole("player")] or "NONE"
	end

	return "NONE"
end

function UnitGroupRoles(unit)
	local role
	-- For 3.3.5 roles are assigned only for LFG, so the API UnitGroupRolesAssigned works only for LFG
	-- For LFG, do not need to use third-party methods to define a role that is not always defined correctly.
	local isTank, isHealer, isDamage = UnitGroupRolesAssigned(unit)
	if ( isTank ) then
		role = "TANK"
	elseif ( isHealer ) then
		role = "HEALER"
	elseif ( isDamage ) then
		role = "DAMAGER"
	else
		-- It makes no sense to use third-party methods to define a role if the role of this class is only one.
		local _,class = UnitClass(unit)
		if ( class == "HUNTER" or class == "ROGUE" or class == "MAGE" or class == "WARLOCK" ) then
			role = "DAMAGER"
		elseif ( LibGT ) then
			role = LibGTConvertRole[LibGT:GetUnitRole(unit)]
		end
	end

	return role or "NONE"
end